<?php
class newswire_controller {
	
	private function url_translit($string) {
		if (function_exists('url_translit')) {
			return url_translit($string);
		}
		
		$db = Main::lib('Db');
		
		$result = $db->select('translit', '`replace_what`, `replace_with`');
				
		if (!($result && $result->num_rows)) {
			return false; 
		}
		
		while($row = $result->fetch_assoc()){
			$translit[$row['replace_what']] = $row['replace_with'];
		}
		
		$string = strtr($string, $translit);
		
		return preg_replace("/[\s]+/", "_", trim(preg_replace("/[\s]+/", ' ', preg_replace('/[^\w ]/i', '', $string))));
	}
	
	public function get_news($lang = 'rus', $limit = 15) {
		
		$newswire_m = Main::model('newswire');
		
		$langs = $newswire_m->get_langs();
		$langs['en'] = 'default';
		$langs['ru'] = 'rus';
		
		if (!in_array($lang, $langs)) {
			return false;
		}
		
		$result = $newswire_m->get_news(array_search($lang, $langs), $limit);
		
		if (!$result) {
			return false;
		}
		
		$news_item_tpl = file_get_contents(TEMPLATE_DIR . '/modules/newswire/news_item.html');
		
		$news_item_tpl_img = file_get_contents(TEMPLATE_DIR . '/modules/newswire/news_item_img.html');
		
		$img_config = array (
			'width' => 160,
			'height' => 120
		);
		
		$news_item_tpl_img = str_replace(array('{width}', '{height}'), array($img_config['width'], $img_config['height']), $news_item_tpl_img);
		
		$html = '';
		
		$search = array('{href}', '{img}', '{title}', '{text}', '{date}', '{section}');
		
		$enabled_news = get_enabled_modules('news');
		$catalog_cat = array();
		$enabled_articles = get_enabled_modules('articles');
		
		$db = Main::lib('Db');
		$pref = $db->getPrefix();
			
		while ($row = $result->fetch_assoc()) {
			if ($row['image']) {
				
				$image = str_replace(
					'{src}', 
					'/admin/modules/flyresize/resize.php?src=' . urlencode($row['image']) . '&w=' . $img_config['width'] . '&h=' . $img_config['height'], 
					$news_item_tpl_img
				);
				
			} else {
				$image = '';
			}
			
			
			
			switch($row['type']) {
				case 'news':
					if (!array_key_exists($row['parent_id_element'], $enabled_news)) {
						continue 2;
					}
					$link = $enabled_news[$row['parent_id_element']] . $row['link'];
				break;
				
				case 'catalog':
			
					if (!array_key_exists($row['parent_id_element'], $catalog_cat)) {
						$result2 = $db->query('
							SELECT
								c.`id`,
								LOWER(w.`' . $lang . '`) as `name_' . $lang . '`
							FROM
								`' . $pref . 'categories` AS `c`
							LEFT JOIN 
								`' . $pref . 'words` AS `w`
							ON 
								w.`string` = c.`string`
							WHERE
								c.`id` = ' . $row['parent_id_element'] . '
						');
						
						if ($result2 && $result2->num_rows) {
							$row2 = $result2->fetch_assoc();
							$catalog_cat[$row2['id']] = $this->url_translit($row2['name_' . $lang]);
						}  else {
							continue 2;
						}
					}
					
					$link = '/' . $catalog_cat[$row['parent_id_element']] . '/' . $row['link'];
					
				break;
				
				case 'article':
					if (!array_key_exists($row['parent_id_element'], $enabled_articles)) {
						continue 2;
					}
					$link = $enabled_articles[$row['parent_id_element']] . $row['link'];
				break;
				
				default:
					continue 2;
			}

			$replace = array($link, $image, $row['title'], $row['description'], date('d.m.Y',$row['date']), '{w:newswire_' . $row['type'] . '}');
			
			$html .= str_replace($search, $replace, $news_item_tpl);
		}
		
		return $html;
	}
	
	public function delete_news(array $data) {
		return Main::model('newswire')->delete_news($data);
	}
	
	public function add_news(array $data) {
		
		$db = Main::lib('Db');
		$pref = $db->getPrefix();
		$data['id'] = (int)$data['id'];
		
		$newswire_m = Main::model('newswire');
		$langs = $newswire_m->get_langs();
		$langs['en'] = 'default';
		$langs['ru'] = 'rus';
		$result = false;
		
		if ($update_id = $newswire_m->check_news($data['type'], $data['id'])) {
			$edit_type = 'update';
		} else {
			$update_id = null;
			$edit_type = 'insert';
		}
		
		//controlers & models 
		switch ($data['type']) {
			case 'catalog':
			
				foreach ($langs as $lang_k => $lang) {
					$fields_lang_items .= '
						, i.`' . $lang . '` AS `text_' . $lang_k . '`
						, LOWER(w.`' . $lang . '`) AS `link_' . $lang_k . '`
						, w.`' . $lang . '` AS `title_' . $lang_k . '`
					';
				}
				
				$result = $db->query('
					SELECT
						i.`id`,
						i.`for`,
						i.`images`,
						i.`date`
						' . $fields_lang_items . '
					FROM 
						`' . $pref . 'items` AS `i`
					LEFT JOIN 
						`' . $pref . 'words` AS `w`
					ON 
						i.`string` = w.`string`
					WHERE
						i.`id` = ' . $data['id'] . '
					LIMIT 1
				');
				
				if ($result && $result->num_rows) {
				
					$item = $result->fetch_assoc();
					
					$data = array(
						'type' => 'catalog',
						'id_element' => $item['id'],
						'parent_id_element' => (int)$item['for'],
						'date' => date('Y-m-d H:i:s',$item['date']),
						'langs' => array()
					);
					
					foreach ($langs as $lang_k => $lang) {
						if (empty($item['link_' . $lang_k])) {//убираем битые ссылки
							unset($langs[$lang_k]); 
						} else {
							$data['langs'][$lang_k] = array(
								'link' =>  $this->url_translit($item['link_' . $lang_k]),
								'title' => mb_substr(trim($item['title_'. $lang_k]), 0, 100),
								'description' => mb_substr(trim(strip_tags($item['text_'. $lang_k])), 0, 252) . '...'
							);
						}
					}
					

					if (count($langs)) {
						if ($delimiter_pos = strpos($item['images'], ';')) {
							$image_name = mb_substr($item['images'], 0, $delimiter_pos);
							$data['image'] = '/images/items/' . $item['for'] . '/' . $item['id'] . '/' . $image_name;
						}
						$result = $newswire_m->edit_news($data, $edit_type, $update_id);	
					} else {
						$result = false;
					}						
					
				}
				break;
				
			case 'article':
				
				foreach ($langs as $lang_k => $lang) {
					$fields_lang_article .= '
						, w.`' . $lang . '` as `title_' . $lang_k . '`
						, a.`' . $lang . '` as `text_' . $lang_k . '`
						, LOWER(w.`' . $lang . '`) as `link_' . $lang_k . '`
					';
				}
				
				$result = $db->query('
					SELECT
						a.`id`,
						a.`for`,
						a.`date`
						' . $fields_lang_article . '
					FROM 
						`' . $pref . 'articles` AS `a`
					LEFT JOIN 
						`' . $pref . 'words` AS `w`
					ON 
						a.`string` = w.`string`
					WHERE
						a.`id` = ' . $data['id'] . '
					LIMIT 1
				');
				
				if ($result && $result->num_rows) {
				
					$article = $result->fetch_assoc();
					
					$data = array(
						'type' => 'article',
						'id_element' => $article['id'],
						'parent_id_element' => $article['for'],
						'date' => $article['date'],
						'langs' => array()
					);
					
					foreach ($langs as $lang_k => $lang) {
						if (empty($article['link_' . $lang_k])) {//убираем битые ссылки
							unset($langs[$lang_k]); 
						} else {
							$data['langs'][$lang_k] = array(
								'link' =>  $this->url_translit($article['link_' . $lang_k]),
								'title' => mb_substr(trim($article['title_'. $lang_k]), 0, 100),
								'description' => mb_substr(trim(strip_tags($article['text_'. $lang_k])), 0, 252) . '...'
							);
							//ишем одно изображение хотябы в одном из языков
							if (!$data['image']) {
								if(preg_match('/<img.*src=["\']{1}([a-za-z0-9\/\.\_]{1,})["\']{1}/ui', $article['text_'. $lang_k], $image)) {
									$data['image'] = $image[1];
								}
							}
						}
					}
					
					if (count($langs)) {
						$result = $newswire_m->edit_news($data, $edit_type, $update_id);	
					} else {
						$result = false;
					}						
					
				}
				break;

			case 'news':
			
				foreach ($langs as $lang_k => $lang) {
					$fields_lang_news .= '
						, w.`' . $lang . '` as `title_' . $lang_k . '`
						, n.`' . $lang . '` as `text_' . $lang_k . '`
						, LOWER(w.`' . $lang . '`) as `link_' . $lang_k . '`
					';
				}

				$result = $db->query('
					SELECT
						n.`id`,
						n.`for`,
						n.`date`
						' . $fields_lang_news . '
					FROM 
						`' . $pref . 'news` AS `n`
					LEFT JOIN 
						`' . $pref . 'words` AS `w`
					ON 
						n.`string` = w.`string`
					WHERE
						n.`id` = ' . $data['id'] . '
					LIMIT 1
				');
				
				if ($result && $result->num_rows) {
					$news = $result->fetch_assoc();
					
					$data = array(
						'type' => 'news',
						'id_element' => $news['id'],
						'parent_id_element' => (int)$news['for'],
						'date' => $news['date'],
						'langs' => array()
					);
					
					
					foreach ($langs as $lang_k => $lang) {
						if (empty($news['link_' . $lang_k])) {//убираем битые ссылки
							unset($langs[$lang_k]); 
						} else {
							$data['langs'][$lang_k] = array(
								'link' =>  $this->url_translit($news['link_' . $lang_k]),
								'title' => mb_substr(trim($news['title_'. $lang_k]), 0, 100),
								'description' => mb_substr(trim(strip_tags($news['text_'. $lang_k])), 0, 252) . '...'
							);
							//ишем одно изображение хотябы в одном из языков
							if (!$data['image']) {
								if(preg_match('/<img.*src=["\']{1}([a-za-z0-9\/\.\_]{1,})["\']{1}/ui', $news['text_'. $lang_k], $image)) {
									$data['image'] = $image[1];
								}
							}
						}
					}
					
					if (count($langs)) {
						$result = $newswire_m->edit_news($data, $edit_type, $update_id);	
					} else {
						$result = false;
					}						
				}
				
				break;
		}
		
		return $result;
	}
	
}
?>