<?php
class newswire_model {
	
	private $langs = array(
		'ru' => 'ru',
		'en' => 'en',
		'es' => 'es'
	);
	
	public function edit_news(array $data, $type_edit, $id = null) {

		if (!isset($data['type'], $data['id_element'], $data['date'], $data['langs'])) {
			return false;
		}
		
		$data_newswire = array(
			'type' => $data['type'],
			'id_element' => (int)$data['id_element'],
			'parent_id_element' => (int)$data['parent_id_element'],
			'date' => $data['date']
		);
		
		if ($data['image']) {
			$data_newswire['image'] = $data['image'];
		}
		
		$transaction = false; 
		$error = false;
		
		$db = Main::lib('Db');
		
		$db->autocommit(false);
		
		switch ($type_edit) {
			case 'update':
			
				if ($db->update('newswire', $data_newswire, '`id` = ' . $id)) {
						
					foreach ($data['langs'] as $lang=>$data) {
					
						$data_lang = array(
							'title' => $data['title'],
							'description' => $data['description'],
							'link' => $data['link']
						);
						
						if (!$db->update('newswire_' . $lang, $data_lang, '`id` = ' . $id)) {
							$error = true;
							break;
						}
						
					}
					
					if ($error) {
						$db->rollback();
					} else {
						$db->commit();
						$transaction = true;
					}
				}
				break;
				
			case 'insert':
				if ($db->insert('newswire', $data_newswire)) {
					
					foreach ($data['langs'] as $lang=>$data) {
					
						$data_lang = array(
							'id' => $db->insert_id,
							'title' => $data['title'],
							'description' => $data['description'],
							'link' => $data['link']
						);
						
						
						if (!$db->insert('newswire_' . $lang, $data_lang)) {
							$error = true;
							break;
						}
						
					}
					
					if ($error) {
						$db->rollback();
					} else {
						$db->commit();
						$transaction = true;
					}
				}
				break;
		}
		
		$db->autocommit(true);
		
		return $transaction;
	}
	
	public function delete_news(array $data) {
		$data['id'] = (int)$data['id'];
		
		if (empty($data['id']) || empty($data['type'])) {
			return false;
		}
		
		$db = Main::lib('Db');
		
		$result = $db->select('newswire', '`id`', '`id_element` = ' . $data['id'] . ' AND `type` = \'' . $db->escape_string($data['type']) . '\'');
		
		if (!($result && $result->num_rows)) {
			return false;
		}
		
		$row = $result->fetch_assoc();
		
		$db->autocommit(false);
		$transaction = false;
		$error = false;
		
		if ($db->delete('newswire', '`id` = ' . $row['id'])) {
			foreach ($this->langs as $lang) {
				if (!$db->delete('newswire_' . $lang, '`id` = ' . $row['id'])) {
					$error = true;
					break;
				}
			}
			
			if ($error) {
				$db->rollback();
			} else {
				$db->commit();
				$transaction = true;
			}
			
		}
		
		$db->autocommit(true);
		
		return $transaction;
	}
	
	public function update_news($id, array $data) {
		$id = (int)$id;
		return $this->edit_news($data, 'update', $id);
	}
	
	public function add_news(array $data) {
		return $this->edit_news($data, 'insert');
	}
	
	
	public function check_news($type, $id_element) {
		$db = Main::lib('Db');
		
		$result = $db->select('newswire', '`id`', '`type` = \'' . $db->escape_string($type) . '\' AND `id_element` = ' . ((int)$id_element));
		
		if ($result && $result->num_rows) {
			$row = $result->fetch_assoc();
			return $row['id'];
		} else {
			return false;
		}
	}
	
	
	public function get_langs() {
		return $this->langs;
	}
	
	public function get_news($lang = 'ru', $limit = 15, $order = 'DESC') {
		if (!in_array($order, array('DESC', 'ASC'))) {
			return false;
		}
		
		$db = Main::lib('Db');
		$pref = $db->getPrefix();
						
		$result = $db->query('
			SELECT
				n.`id`,
				n.`type`,
				n.`id_element`,
				n.`parent_id_element`,
				n.`image`,
				UNIX_TIMESTAMP(n.`date`) AS `date`,
				n_r.`title`,
				n_r.`description`,
				n_r.`link`
			FROM 
				`' . $pref . 'newswire` AS `n`
			RIGHT JOIN 
				`' . $pref . 'newswire_' . $lang . '` AS `n_r`
			ON 
				n.`id` = n_r.`id`
			ORDER BY 
				n.`date` ' . $order . '
			LIMIT 
				' . $limit . '
		');
		
		if ($result && $result->num_rows) {
			return $result;
		} else {
			return false;
		}
	}
}
?>